﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace CustomControls {
  /// <summary>
  /// Interaction logic for BindingWindow.xaml
  /// </summary>
  public partial class BindingWindow : Window {
    public BindingWindow() {
      InitializeComponent();

      DataContext = new BindingViewModel();
    }

    private void cbProduct_ValueChanged(object sender, RoutedPropertyChangedEventArgs<object> e) {

    }

    private void btnClearProduct_Click(object sender, RoutedEventArgs e) {
      cbProduct.Value = null;
    }

  }

  public class BindingViewModel : BaseObject {

    private ProductCategory _SelectedCategory;
    private Product _SelectedProduct;

    public BindingViewModel() {
      Categories = ProductCategory.GetTestData();
      SelectedCategory = Categories[0];
    }

    public ObservableCollection<ProductCategory> Categories { get; private set; }

    public ProductCategory SelectedCategory {
      get { return _SelectedCategory; }
      set { 
        SetProperty<ProductCategory>("SelectedCategory", value, ref _SelectedCategory);
        SelectedProduct = _SelectedCategory.Items.Count > 0 ? _SelectedCategory.Items[0] : null;
      }
    }

    public Product SelectedProduct {
      get { return _SelectedProduct; }
      set { SetProperty<Product>("SelectedProduct", value, ref _SelectedProduct); }
    }
  }

  public class ProductCategory : BaseObject {
    private int _Id;
    private string _Code;
    private ObservableCollection<Product> _Items;

    public ProductCategory(int id, string code) {
      _Id = id;
      _Code = code;
      _Items = new ObservableCollection<Product>();
    }

    public int Id {
      get { return _Id; }
      set { SetProperty<int>("Id", value, ref _Id); }
    }

    public string Code {
      get { return _Code; }
      set { SetProperty<string>("Code", value, ref _Code); }
    }

    public ObservableCollection<Product> Items {
      get { return _Items; }
      set { SetProperty<ObservableCollection<Product>>("Items", value, ref _Items); }
    }

    public static ObservableCollection<ProductCategory> GetTestData() {
      var result = new ObservableCollection<ProductCategory>();

      var productCategory = new ProductCategory(1, "Category A");
      productCategory.Items.Add(new Product(1, "Product A1"));
      productCategory.Items.Add(new Product(2, "Product A2"));
      productCategory.Items.Add(new Product(3, "Product A3"));
      result.Add(productCategory);

      productCategory = new ProductCategory(2, "Category B");
      productCategory.Items.Add(new Product(4, "Product B1"));
      productCategory.Items.Add(new Product(5, "Product B2"));
      result.Add(productCategory);

      productCategory = new ProductCategory(3, "Category C");
      result.Add(productCategory);

      return result;
    }

  }

  public class Product : BaseObject {
    private int _Id;
    private string _Code;

    public Product(int id, string code) {
      _Id = id;
      _Code = code;
    }

    public int Id {
      get { return _Id; }
      set { SetProperty<int>("Id", value, ref _Id); }
    }

    public string Code {
      get { return _Code; }
      set { SetProperty<string>("Code", value, ref _Code); }
    }

  }
}
